searchData={"items":[{"type":"module","title":"StateMachine","doc":"StateMachine package implements state machine abstraction.\nIt supports Ecto out of the box and can work as both\ndata structure and a process powered by gen_statem.\n\nCheck out the [article](https://dev.to/youroff/state-machines-for-business-np8) for motivation.\n\nHere's an example of a simple state machine created with this package:\n\n    defmodule Cat do\n      use StateMachine\n\n      defstruct [:name, :state, hungry: true]\n\n      defmachine field: :state do\n        state :asleep\n        state :awake\n        state :playing\n        state :eating, after_enter: &Cat.feed_up/1\n\n        event :wake do\n          transition from: :asleep, to: :awake\n        end\n\n        event :give_a_mouse do\n          transition from: :awake, to: :playing, unless: &Cat.hungry/1\n          transition from: :awake, to: :eating, if: &Cat.hungry/1\n          transition from: :playing, to: :eating\n        end\n\n        event :pet do\n          transition from: [:eating, :awake], to: :playing\n        end\n\n        event :sing_a_lullaby do\n          transition from: :awake, to: :asleep\n          transition from: :playing, to: :asleep\n        end\n      end\n\n      def hungry(cat) do\n        cat.hungry\n      end\n\n      def feed_up(cat) do\n        {:ok, %{cat | hungry: false}}\n      end\n    end\n\nAnd later use it like this:\n\n    cat = %Cat{name: \"Thomas\", state: :asleep}\n\n    {:ok, %Cat{state: :awake}} = Cat.trigger(cat, :wake)","ref":"StateMachine.html"},{"type":"module","title":"Features - StateMachine","doc":"* Validation of state machine definition at compile time\n* Full support for callbacks (on states, events and transitions) and guards (on events and transitions)\n* Optional payload can be supplied with the event\n* One-line conversion to a state machine as a process (powered by gen_statem)\n* With Ecto support activated every transition is wrapped in transaction\n* With Ecto support activated the Ecto.Type implementation is generated automatically","ref":"StateMachine.html#module-features"},{"type":"type","title":"StateMachine.t/1","doc":"","ref":"StateMachine.html#t:t/1"},{"type":"module","title":"StateMachine.Callback","doc":"Callback defines a captured function, that can be called\nin a various points of a State Machine's lifecycle.\nDepending on return type (shape) of the callback,\nit can update the context or the model, stop the transition with error, or be ignored.","ref":"StateMachine.Callback.html"},{"type":"function","title":"StateMachine.Callback.apply_callback/3","doc":"Applying a single callback. There are three types of callbacks supported:","ref":"StateMachine.Callback.html#apply_callback/3"},{"type":"function","title":"Unary (unary_t type) - StateMachine.Callback.apply_callback/3","doc":"A unary callback, receiving a model struct and that can be updated\nin current conetxt based on shape of the return:\n  * `{:ok, model}` — replaces model in the context\n  * `{:error, e}` — stops the transition with a given error\n  * `any` — doesn't have any effect on the context","ref":"StateMachine.Callback.html#apply_callback/3-unary-unary_t-type"},{"type":"function","title":"Binary (binary_t type) - StateMachine.Callback.apply_callback/3","doc":"A binary callback, receiving a model struct and a context.\nEither can be updated depending on the shape of the return:\n  * `{:ok, context}` — replaces context completely\n  * `{:ok, model}` — replaces model in the context\n  * `{:error, e}` — stops the transition with a given error\n  * `any` — doesn't have any effect on the context","ref":"StateMachine.Callback.html#apply_callback/3-binary-binary_t-type"},{"type":"function","title":"Side effects  (side_effect_t type) - StateMachine.Callback.apply_callback/3","doc":"A type of callback that does not expect any input and potentially produces a side effect.\nAny return value is ignored, except for `{:error, e}` that stops the transition with a given error.","ref":"StateMachine.Callback.html#apply_callback/3-side-effects-side_effect_t-type"},{"type":"function","title":"StateMachine.Callback.apply_chain/3","doc":"Applying a chain of callbacks. Application only happens if `status` hasn't been changed.","ref":"StateMachine.Callback.html#apply_chain/3"},{"type":"type","title":"StateMachine.Callback.binary_t/1","doc":"","ref":"StateMachine.Callback.html#t:binary_t/1"},{"type":"type","title":"StateMachine.Callback.side_effect_t/0","doc":"","ref":"StateMachine.Callback.html#t:side_effect_t/0"},{"type":"type","title":"StateMachine.Callback.t/1","doc":"","ref":"StateMachine.Callback.html#t:t/1"},{"type":"type","title":"StateMachine.Callback.unary_t/1","doc":"","ref":"StateMachine.Callback.html#t:unary_t/1"},{"type":"module","title":"StateMachine.Context","doc":"Context is a container for all metadata supporting the transition.\nNormally users should not have anything to do with Contexts,\nhowever some public functions might expose it. For example, user can\n(but discuraged) manipulate the Context in callbacks.","ref":"StateMachine.Context.html"},{"type":"function","title":"StateMachine.Context.build/2","doc":"Builds a fresh context based on State Machine definition and a model struct.","ref":"StateMachine.Context.html#build/2"},{"type":"type","title":"StateMachine.Context.t/1","doc":"","ref":"StateMachine.Context.html#t:t/1"},{"type":"module","title":"StateMachine.DSL","doc":"These macros help generating State Machine definition in a given Module.","ref":"StateMachine.DSL.html"},{"type":"macro","title":"StateMachine.DSL.define_gen_statem/0","doc":"Experimental macro to generate GenStatem definition. See source...","ref":"StateMachine.DSL.html#define_gen_statem/0"},{"type":"macro","title":"StateMachine.DSL.defmachine/2","doc":"Creates a main block for a State Machine definition. Compile time checks and validations help\nensuring correct usage of this and other macros.\n\n    use StateMachine\n\n    defmachine field: :custom do\n      # ... states, events, transitions\n    end","ref":"StateMachine.DSL.html#defmachine/2"},{"type":"macro","title":"Options - StateMachine.DSL.defmachine/2","doc":"* `:field` - what field in the model structure holds the state value. Default: `:state`.\n  * `:repo` - the Ecto Repo module. If provider, the support for Ecto is activated, including\n    state getter/setters, Ecto.Type generation and custom `trigger` function.\n  * `:state_type` - name for `Ecto.Type` implementation for State type. Generated inside of the module namespace.\n    If you state machine is `App.StateMachine`, then `Ecto.Type` implementation will be\n    accessible on `App.StateMachine.(state_type)`. Default: `StateType`.\n  * `:event_type` - name for `Ecto.Type` implementation for Event type. Default: `EventType`.","ref":"StateMachine.DSL.html#defmachine/2-options"},{"type":"macro","title":"StateMachine.DSL.event/3","doc":"Creates an Event record which encapsulates one or more Transitions.\nConceptually an Event is an external signal to change the state.\nEvents can be accompanied by Guards — additional conditions, allowing\nfor implementing more complex logic than supported by theoretical\nstate machines.\n\nUsing guards allow for transitions from one to the multiple states,\nbased on condition.\n\n\n    defmachine do\n      # for state defs, see `state`\n\n      event :wake_up, if: &Bedroom.slept_eight_hours?/1 do\n        # for transition defs, see `transition`\n      end\n    end","ref":"StateMachine.DSL.html#event/3"},{"type":"macro","title":"Options - StateMachine.DSL.event/3","doc":"* `:before` - run the callback before the event.\n  * `:after` - run the callback after the event.\n  * `:if` - positive guard, must return `true` to proceed.\n  * `:unless` - negative guard, must return `false` to proceed.\n  * `:passthrough` - makes the event always allowed, in absense of a custom transition matching the current state and passing guards,\n    it creates a noop transition back to the original state.","ref":"StateMachine.DSL.html#event/3-options"},{"type":"macro","title":"StateMachine.DSL.state/2","doc":"Creates a State record with any atom as a name.\nSupports defining callbacks for before/after leaving/entering given state.\nFor detailed description of Callbacks, see the module documentation,\nbut currently only fully qualified function capture is supported: `&Module.fun/arity`\n\n    defmachine do\n      state :sleeping, after_leave: &Kitchen.brew_coffee/1\n      state :working, before_enter: [&Commute.drive/1, &Commute.grab_a_newspaper/1]\n    end","ref":"StateMachine.DSL.html#state/2"},{"type":"macro","title":"Options - StateMachine.DSL.state/2","doc":"* `:before_leave` - run the callback before leaving this state.\n  * `:after_leave` - run the callback after leaving this state.\n  * `:before_enter` - run the callback before entering this state.\n  * `:after_enter` - run the callback after entering this state.","ref":"StateMachine.DSL.html#state/2-options"},{"type":"macro","title":"StateMachine.DSL.transition/1","doc":"Defines a Transition, a path from one or more states to a single state.\nThe same Event can contain all crazy combinations of transitions,\nbut the resolution happens from top to bottom, from left to right.\nFirst matching transition will run.\n\n\n    defmachine do\n      # for state defs, see `state`\n\n      event :wake_up do\n        transition from: :sleeping, to: :sleeping,\n          if: &Calendar.weekend?/0,\n          unless: &Bedroom.slept_ten_hours?/1\n        transition from: :sleeping, to: :awake\n      end\n    end","ref":"StateMachine.DSL.html#transition/1"},{"type":"macro","title":"Options - StateMachine.DSL.transition/1","doc":"* `:before` - run the callback before the transition.\n  * `:after` - run the callback after the transition.\n  * `:if` - positive guard, must return `true` to proceed.\n  * `:unless` - negative guard, must return `false` to proceed.","ref":"StateMachine.DSL.html#transition/1-options"},{"type":"module","title":"StateMachine.Ecto","doc":"This addition makes StateMachine fully compatible with Ecto.\n\nState setter and getter are abstracted in order to provide a way to update a state\nin the middle of transition for a various types of models. With Ecto, we call `change() |> Repo.update`.\nWe also wrap every event in transaction, which is rolled back if transition fails to finish.\nThis unlocks a lot of beautiful effects. For example, you can enqueue some tasks into db-powered queue in callbacks,\nand if transition failes, those tasks will naturally disappear.","ref":"StateMachine.Ecto.html"},{"type":"module","title":"Usage - StateMachine.Ecto","doc":"To use Ecto, simply pass `repo` param to `defmachine`, you can optionally pass a name of the `Ecto.Type`\nimplementation, that will be generated automatically under state machine namespace:\n\n\n    defmodule EctoMachine do\n      use StateMachine\n\n      defmachine field: :state, repo: TestApp.Repo, ecto_type: CustomMod do\n        state :resting\n        state :working\n\n        # ...\n      end\n    end\n\nIn your schema you can refer to state type as `EctoMachine.CustomMod`, with `ecto_type` omitted\nit would generate `EctoMachine.StateType`. This custom type is needed to transparently use atoms as states.","ref":"StateMachine.Ecto.html#module-usage"},{"type":"macro","title":"StateMachine.Ecto.define_ecto_type/1","doc":"This macro defines an Ecto.Type implementation inside of StateMachine namespace.\nThe default name is `StateType`, but you can supply any module name.\nThe purpose of this is to be able to cast string into atom and back safely,\nvalidating it against StateMachine defition.","ref":"StateMachine.Ecto.html#define_ecto_type/1"},{"type":"macro","title":"StateMachine.Ecto.define_enum/1","doc":"","ref":"StateMachine.Ecto.html#define_enum/1"},{"type":"module","title":"StateMachine.Event","doc":"Event is a container for transitions. It is identified by name (atom) and can contain arbitrary number of transtitions.\n\nOne important thing is that it's disallowed to have more than one unguarded transition from the state, since this\nwould introduce a \"non-determinism\" (or rather just discard the latter transition). We loosely allow guarded transitions\nfrom the same state, but it doesn't guarantee anything: if guards always return true, we're back to where we were before.","ref":"StateMachine.Event.html"},{"type":"function","title":"StateMachine.Event.callback/2","doc":"Private function for running Event callbacks.","ref":"StateMachine.Event.html#callback/2"},{"type":"type","title":"StateMachine.Event.callback_pos/0","doc":"","ref":"StateMachine.Event.html#t:callback_pos/0"},{"type":"function","title":"StateMachine.Event.is_allowed?/2","doc":"Checks if the event is allowed in the current context. First it makes sure that all guards\nof the event return `true`, then it scans transitions for the matching one. Match is determined\nby the source state and passing of all guards as well.","ref":"StateMachine.Event.html#is_allowed?/2"},{"type":"type","title":"StateMachine.Event.t/1","doc":"","ref":"StateMachine.Event.html#t:t/1"},{"type":"function","title":"StateMachine.Event.trigger/3","doc":"This is an entry point for transition. By running this function with populated context, event\nand optional payload, you tell state machine to try to move to the next state.\nIt returns an updated context.","ref":"StateMachine.Event.html#trigger/3"},{"type":"module","title":"StateMachine.Guard","doc":"Guards are functions that help to decide whether it's allowed to proceed with Event\nor Transition. They might serve different purposes:\n\n* Preventing a state machine from getting into a state unless some criteria met\n* Creating event with multiple target states with a single source state\n\nGuards should not have any side-effects, cause they are getting run no matter\nif transition is successful or not, and also to determine the list of possible events\nfor a certain state.","ref":"StateMachine.Guard.html"},{"type":"function","title":"StateMachine.Guard.check/2","doc":"Check runs guards associated with given Event or Transition\nand returns true if all passed. First argument of the guard is a model,\nsecond argument is the context.","ref":"StateMachine.Guard.html#check/2"},{"type":"function","title":"StateMachine.Guard.prepare/1","doc":"Unifies `if` and `unless` guards into a single stream of guards.","ref":"StateMachine.Guard.html#prepare/1"},{"type":"type","title":"StateMachine.Guard.t/1","doc":"","ref":"StateMachine.Guard.html#t:t/1"},{"type":"module","title":"StateMachine.Introspection","doc":"Introspection functions allow you to collect and analyze meta information about state machine in runtime.\nFor example, you might want to send the full list of states or events to the front-end, so that appropriate\ncontrol or scale could be rendered. It is also possible to generate the list of currently allowed events,\nbut be careful, this operation can be expensive if you use guards with side-effects.","ref":"StateMachine.Introspection.html"},{"type":"function","title":"StateMachine.Introspection.all_events/1","doc":"Returns a list of all events of the supplied state machine definition.","ref":"StateMachine.Introspection.html#all_events/1"},{"type":"function","title":"StateMachine.Introspection.all_states/1","doc":"Returns a list of all states of the supplied state machine definition.","ref":"StateMachine.Introspection.html#all_states/1"},{"type":"function","title":"StateMachine.Introspection.allowed_events/1","doc":"Returns a list of available events. This function takes a context with populated model,\nthis is necessary to get a current state.","ref":"StateMachine.Introspection.html#allowed_events/1"},{"type":"behaviour","title":"StateMachine.State","doc":"State module provides a structure describing a state in the state machine definition.\nIt stores state name along with various callbacks to run before/after leaving/entering a state.\nAnother purpose of the module is to define a behaviour for state get/setters along with the default implementation.\n\nThe state get/setters for basic structures and Ecto records are provided out of the box.","ref":"StateMachine.State.html"},{"type":"function","title":"StateMachine.State.callback/2","doc":"Private function for running state callbacks.","ref":"StateMachine.State.html#callback/2"},{"type":"type","title":"StateMachine.State.callback_pos/0","doc":"","ref":"StateMachine.State.html#t:callback_pos/0"},{"type":"function","title":"StateMachine.State.get/1","doc":"Default implementation of a state getter.","ref":"StateMachine.State.html#get/1"},{"type":"callback","title":"StateMachine.State.get/1","doc":"","ref":"StateMachine.State.html#c:get/1"},{"type":"function","title":"StateMachine.State.set/2","doc":"Default implementation of a state setter.","ref":"StateMachine.State.html#set/2"},{"type":"callback","title":"StateMachine.State.set/2","doc":"","ref":"StateMachine.State.html#c:set/2"},{"type":"type","title":"StateMachine.State.t/1","doc":"","ref":"StateMachine.State.html#t:t/1"},{"type":"module","title":"StateMachine.Transition","doc":"Transition module gathers together all of the actions that happen\naround transition from the old state to the new state in response to an event.","ref":"StateMachine.Transition.html"},{"type":"function","title":"StateMachine.Transition.callback/2","doc":"Private function for running Transition callbacks.","ref":"StateMachine.Transition.html#callback/2"},{"type":"type","title":"StateMachine.Transition.callback_pos/0","doc":"","ref":"StateMachine.Transition.html#t:callback_pos/0"},{"type":"function","title":"StateMachine.Transition.finalize/1","doc":"Private function sets status to :done, unless it has failed before.","ref":"StateMachine.Transition.html#finalize/1"},{"type":"function","title":"StateMachine.Transition.is_allowed?/2","doc":"Checks if the transition is allowed in the current context. Returns boolean.","ref":"StateMachine.Transition.html#is_allowed?/2"},{"type":"function","title":"StateMachine.Transition.loop?/1","doc":"True if transition is a loop, i.e. doesn't change state.","ref":"StateMachine.Transition.html#loop?/1"},{"type":"function","title":"StateMachine.Transition.passthrough/1","doc":"","ref":"StateMachine.Transition.html#passthrough/1"},{"type":"function","title":"StateMachine.Transition.run/1","doc":"Given populated context and Transition structure,\nsequentially runs all callbacks along with actual state update:\n\n* before(event)\n* before(transition)\n* before_leave(state)\n* before_enter(state)\n* *** (state update) ***\n* after_leave(state)\n* after_enter(state)\n* after(transition)\n* after(event)\n\nIf any of the callbacks fails, all sequential ops are cancelled.","ref":"StateMachine.Transition.html#run/1"},{"type":"type","title":"StateMachine.Transition.t/1","doc":"","ref":"StateMachine.Transition.html#t:t/1"},{"type":"function","title":"StateMachine.Transition.update_state/1","doc":"Private function for updating state.","ref":"StateMachine.Transition.html#update_state/1"},{"type":"module","title":"StateMachine.Utils","doc":"","ref":"StateMachine.Utils.html"},{"type":"function","title":"StateMachine.Utils.keyword_splat/3","doc":"","ref":"StateMachine.Utils.html#keyword_splat/3"},{"type":"function","title":"StateMachine.Utils.normalize_funciton/2","doc":"","ref":"StateMachine.Utils.html#normalize_funciton/2"},{"type":"function","title":"StateMachine.Utils.normalize_function/2","doc":"","ref":"StateMachine.Utils.html#normalize_function/2"},{"type":"module","title":"StateMachine.Validation","doc":"StateMachine supports automatic validation on compile time.\nIt makes its best effort to ensure determinism (to some degree).\nIt checks that all used states were previously defined.\n\nTODO: Check unreachable states?","ref":"StateMachine.Validation.html"},{"type":"function","title":"StateMachine.Validation.validate_all/1","doc":"","ref":"StateMachine.Validation.html#validate_all/1"},{"type":"function","title":"StateMachine.Validation.validate_states_in_transitions/1","doc":"Validates presense of states used in transitions.","ref":"StateMachine.Validation.html#validate_states_in_transitions/1"},{"type":"function","title":"StateMachine.Validation.validate_transitions_determinism/1","doc":"Validates that no more than one unguarded transition is originated in every state.","ref":"StateMachine.Validation.html#validate_transitions_determinism/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}